@@ -41,7 +41,9 @@
 
 #include <mm-errors-types.h>
 #include <mm-gdbus-manager.h>
-#include <mm-gdbus-test.h>
+#if defined WITH_TESTS
+# include <mm-gdbus-test.h>
+#endif
 
 #include "mm-context.h"
 #include "mm-base-manager.h"
@@ -66,9 +68,11 @@ enum {
     PROP_CONNECTION,
     PROP_AUTO_SCAN,
     PROP_FILTER_POLICY,
-    PROP_ENABLE_TEST,
     PROP_PLUGIN_DIR,
     PROP_INITIAL_KERNEL_EVENTS,
+#if defined WITH_TESTS
+    PROP_ENABLE_TEST,
+#endif
     LAST_PROP
 };
 
@@ -79,8 +83,6 @@ struct _MMBaseManagerPrivate {
     gboolean auto_scan;
     /* Filter policy (mask of enabled rules) */
     MMFilterRule filter_policy;
-    /* Whether the test interface is enabled */
-    gboolean enable_test;
     /* Path to look for plugins */
     gchar *plugin_dir;
     /* Path to the list of initial kernel events */
@@ -99,8 +101,12 @@ struct _MMBaseManagerPrivate {
     /* The map of inhibited devices */
     GHashTable *inhibited_devices;
 
+#if defined WITH_TESTS
+    /* Whether the test interface is enabled */
+    gboolean enable_test;
     /* The Test interface support */
     MmGdbusTest *test_skeleton;
+#endif
 
 #if defined WITH_UDEV
     /* The UDev client */
@@ -1099,15 +1105,57 @@ remove_device_inhibition (MMBaseManager *self,
     info->port_infos = NULL;
     g_hash_table_remove (self->priv->inhibited_devices, uid);
 
+    /* If any port info exists, we require explicit port probing that will be
+     * triggered via the artificial port notifications emitted with the
+     * device_added() calls */
     if (port_infos) {
         GList *l;
 
-        /* Note that a device can only be inhibited if it had an existing
-         * modem exposed in the bus. And so, inhibition can only be placed
-         * AFTER all port probes have finished for a given device. This means
-         * that we either have a device tracked, or we have a list of port
-         * infos. Both at the same time should never happen. */
-        g_assert (!device);
+        /* A device may exist at this point if e.g. not all ports were
+         * removed during the inhibition (i.e. the MMDevice was never fully
+         * removed) and new ports were then added while inhibited. In this
+         * case, we must fake all ports going away so that the MMDevice gets
+         * completely removed, otherwise the plugin manager won't start a new
+         * device probing task and therefore no port probing tasks. */
+        if (device) {
+            GList *readded_port_infos = NULL;
+            GList *leftover_ports;
+
+            /* Create a new list of inhibited device port infos from the existing
+             * port probes */
+            leftover_ports = mm_device_peek_port_probe_list (device);
+            for (l = leftover_ports; l; l = g_list_next (l)) {
+                InhibitedDevicePortInfo *port_info;
+                MMPortProbe             *port_probe;
+
+                port_probe = MM_PORT_PROBE (l->data);
+
+                port_info = g_slice_new0 (InhibitedDevicePortInfo);
+                port_info->kernel_port = mm_port_probe_get_port (port_probe);
+                port_info->manual_scan = TRUE;
+                readded_port_infos = g_list_append (readded_port_infos, port_info);
+            }
+
+            /* Now, explicitly request to remove all ports, the device should go
+             * away as well while doing so. */
+            for (l = readded_port_infos; l; l = g_list_next (l)) {
+                InhibitedDevicePortInfo *port_info;
+
+                port_info = (InhibitedDevicePortInfo *)(l->data);
+                mm_obj_info (self, "fake releasing port %s/%s during uninhibition...",
+                             mm_kernel_device_get_subsystem (port_info->kernel_port),
+                             mm_kernel_device_get_name (port_info->kernel_port));
+                device_removed (self,
+                                mm_kernel_device_get_subsystem (port_info->kernel_port),
+                                mm_kernel_device_get_name (port_info->kernel_port));
+            }
+
+            /* At this point, the device should have gone completely */
+            g_assert (!find_device_by_physdev_uid (self, uid));
+
+            /* Added the ports to re-add in the pending list */
+            port_infos = g_list_concat (port_infos, readded_port_infos);
+        }
 
         /* Report as added all port infos that we had tracked while the
          * device was inhibited. We can only report the added port after
@@ -1120,10 +1168,12 @@ remove_device_inhibition (MMBaseManager *self,
             device_added (self, port_info->kernel_port, FALSE, port_info->manual_scan);
         }
         g_list_free_full (port_infos, (GDestroyNotify)inhibited_device_port_info_free);
+        return;
     }
+
     /* The device may be totally gone from the system while we were
      * keeping the inhibition, so do not error out if not found. */
-    else if (device) {
+    if (device) {
         GError *error = NULL;
 
         /* Uninhibit device, which will create and expose the modem object */
@@ -1297,6 +1347,8 @@ handle_inhibit_device (MmGdbusOrgFreedesktopModemManager1 *manager,
 /*****************************************************************************/
 /* Test profile setup */
 
+#if defined WITH_TESTS
+
 static gboolean
 handle_set_profile (MmGdbusTest *skeleton,
                     GDBusMethodInvocation *invocation,
@@ -1358,6 +1410,8 @@ out:
     return TRUE;
 }
 
+#endif
+
 /*****************************************************************************/
 
 static gchar *
@@ -1374,7 +1428,9 @@ mm_base_manager_new (GDBusConnection  *connection,
                      gboolean          auto_scan,
                      MMFilterRule      filter_policy,
                      const gchar      *initial_kernel_events,
+#if defined WITH_TESTS
                      gboolean          enable_test,
+#endif
                      GError          **error)
 {
     g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
@@ -1387,8 +1443,10 @@ mm_base_manager_new (GDBusConnection  *connection,
                            MM_BASE_MANAGER_AUTO_SCAN,             auto_scan,
                            MM_BASE_MANAGER_FILTER_POLICY,         filter_policy,
                            MM_BASE_MANAGER_INITIAL_KERNEL_EVENTS, initial_kernel_events,
-                           MM_BASE_MANAGER_ENABLE_TEST,           enable_test,
                            "version",                             MM_DIST_VERSION,
+#if defined WITH_TESTS
+                           MM_BASE_MANAGER_ENABLE_TEST,           enable_test,
+#endif
                            NULL);
 }
 
@@ -1415,11 +1473,13 @@ set_property (GObject      *object,
                 mm_obj_dbg (self, "stopping connection in object manager server");
                 g_dbus_object_manager_server_set_connection (self->priv->object_manager, NULL);
             }
+#if defined WITH_TESTS
             if (self->priv->test_skeleton &&
                 g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (self->priv->test_skeleton))) {
                 mm_obj_dbg (self, "stopping connection in test skeleton");
                 g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON (self->priv->test_skeleton));
             }
+#endif
         }
         break;
     }
@@ -1429,9 +1489,6 @@ set_property (GObject      *object,
     case PROP_FILTER_POLICY:
         self->priv->filter_policy = g_value_get_flags (value);
         break;
-    case PROP_ENABLE_TEST:
-        self->priv->enable_test = g_value_get_boolean (value);
-        break;
     case PROP_PLUGIN_DIR:
         g_free (self->priv->plugin_dir);
         self->priv->plugin_dir = g_value_dup_string (value);
@@ -1440,6 +1497,11 @@ set_property (GObject      *object,
         g_free (self->priv->initial_kernel_events);
         self->priv->initial_kernel_events = g_value_dup_string (value);
         break;
+#if defined WITH_TESTS
+    case PROP_ENABLE_TEST:
+        self->priv->enable_test = g_value_get_boolean (value);
+        break;
+#endif
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1464,15 +1526,17 @@ get_property (GObject    *object,
     case PROP_FILTER_POLICY:
         g_value_set_flags (value, self->priv->filter_policy);
         break;
-    case PROP_ENABLE_TEST:
-        g_value_set_boolean (value, self->priv->enable_test);
-        break;
     case PROP_PLUGIN_DIR:
         g_value_set_string (value, self->priv->plugin_dir);
         break;
     case PROP_INITIAL_KERNEL_EVENTS:
         g_value_set_string (value, self->priv->initial_kernel_events);
         break;
+#if defined WITH_TESTS
+    case PROP_ENABLE_TEST:
+        g_value_set_boolean (value, self->priv->enable_test);
+        break;
+#endif
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1498,8 +1562,10 @@ mm_base_manager_init (MMBaseManager *self)
     /* By default, enable autoscan */
     self->priv->auto_scan = TRUE;
 
+#if defined WITH_TESTS
     /* By default, no test interface */
     self->priv->enable_test = FALSE;
+#endif
 
     /* Setup Object Manager Server */
     self->priv->object_manager = g_dbus_object_manager_server_new (MM_DBUS_PATH);
@@ -1565,6 +1631,7 @@ initable_init (GInitable     *initable,
     g_dbus_object_manager_server_set_connection (self->priv->object_manager,
                                                  self->priv->connection);
 
+#if defined WITH_TESTS
     /* Setup the Test skeleton and export the interface */
     if (self->priv->enable_test) {
         self->priv->test_skeleton = mm_gdbus_test_skeleton_new ();
@@ -1578,6 +1645,7 @@ initable_init (GInitable     *initable,
                                                error))
             return FALSE;
     }
+#endif
 
     /* All good */
     return TRUE;
@@ -1613,8 +1681,10 @@ finalize (GObject *object)
     if (self->priv->object_manager)
         g_object_unref (self->priv->object_manager);
 
+#if defined WITH_TESTS
     if (self->priv->test_skeleton)
         g_object_unref (self->priv->test_skeleton);
+#endif
 
     if (self->priv->connection)
         g_object_unref (self->priv->connection);
@@ -1678,14 +1748,6 @@ mm_base_manager_class_init (MMBaseManagerClass *manager_class)
                             MM_FILTER_RULE_NONE,
                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-    g_object_class_install_property
-        (object_class, PROP_ENABLE_TEST,
-         g_param_spec_boolean (MM_BASE_MANAGER_ENABLE_TEST,
-                               "Enable tests",
-                               "Enable the Test interface",
-                               FALSE,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
     g_object_class_install_property
         (object_class, PROP_PLUGIN_DIR,
          g_param_spec_string (MM_BASE_MANAGER_PLUGIN_DIR,
@@ -1701,4 +1763,14 @@ mm_base_manager_class_init (MMBaseManagerClass *manager_class)
                               "Path to a file with the list of initial kernel events",
                               NULL,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+#if defined WITH_TESTS
+    g_object_class_install_property
+        (object_class, PROP_ENABLE_TEST,
+         g_param_spec_boolean (MM_BASE_MANAGER_ENABLE_TEST,
+                               "Enable tests",
+                               "Enable the Test interface",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+#endif
 }
