@@ -146,6 +146,18 @@ mm_base_modem_get_dbus_id (MMBaseModem *self)
 
 /******************************************************************************/
 
+static void
+port_removed_cb (MMPort      *port,
+                 MMBaseModem *self)
+{
+    /* We have to do a full re-probe here because simply reopening the device
+     * and restarting proxy would leave us without proper notifications. */
+    mm_obj_info (self, "port '%s' no longer controllable, reprobing",
+                 mm_port_get_device (MM_PORT (port)));
+    self->priv->reprobe = TRUE;
+    g_cancellable_cancel (self->priv->cancellable);
+}
+
 static void
 port_timed_out_cb (MMPort       *port,
                    guint         n_consecutive_timeouts,
@@ -322,10 +334,17 @@ base_modem_internal_grab_port (MMBaseModem         *self,
     const gchar      *subsys;
     const gchar      *name;
     g_autofree gchar *key = NULL;
+    gboolean          port_monitoring = FALSE;
 
     subsys = mm_kernel_device_get_subsystem (kernel_device);
     name   = mm_kernel_device_get_name      (kernel_device);
 
+    if (!self->priv->ports || (link_port && !self->priv->link_ports)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Cannot add port '%s/%s', no ports table", subsys, name);
+        return NULL;
+    }
+
     /* Check whether we already have it stored */
     key  = g_strdup_printf ("%s%s", subsys, name);
     port = g_hash_table_lookup (self->priv->ports, key);
@@ -362,35 +381,37 @@ base_modem_internal_grab_port (MMBaseModem         *self,
         return NULL;
     }
 
-    /* Setup consecutive timeout watcher in all control ports */
-    if (self->priv->max_timeouts > 0) {
-        gboolean timeout_monitoring = FALSE;
-
-        if (MM_IS_PORT_SERIAL_AT (port)) {
-            mm_obj_dbg (port, "timeout monitoring enabled in AT port");
-            timeout_monitoring = TRUE;
-        } else if (MM_IS_PORT_SERIAL_QCDM (port)) {
-            mm_obj_dbg (port, "timeout monitoring enabled in QCDM port");
-            timeout_monitoring = TRUE;
-        }
+    /* Setup consecutive ports and removal watchers in all control ports */
+    if (MM_IS_PORT_SERIAL_AT (port)) {
+        mm_obj_dbg (port, "port monitoring enabled in AT port");
+        port_monitoring = TRUE;
+    } else if (MM_IS_PORT_SERIAL_QCDM (port)) {
+        mm_obj_dbg (port, "port monitoring enabled in QCDM port");
+        port_monitoring = TRUE;
+    }
 #if defined WITH_QMI
-        else if (MM_IS_PORT_QMI (port)) {
-            mm_obj_dbg (port, "timeout monitoring enabled in QMI port");
-            timeout_monitoring = TRUE;
-        }
+    else if (MM_IS_PORT_QMI (port)) {
+        mm_obj_dbg (port, "port monitoring enabled in QMI port");
+        port_monitoring = TRUE;
+    }
 #endif
 #if defined WITH_MBIM
-        else if (MM_IS_PORT_MBIM (port)) {
-            mm_obj_dbg (port, "timeout monitoring enabled in MBIM port");
-            timeout_monitoring = TRUE;
-        }
+    else if (MM_IS_PORT_MBIM (port)) {
+        mm_obj_dbg (port, "port monitoring enabled in MBIM port");
+        port_monitoring = TRUE;
+    }
 #endif
 
-        if (timeout_monitoring)
+    if (port_monitoring) {
+        if (self->priv->max_timeouts > 0)
             g_signal_connect (port,
                               MM_PORT_SIGNAL_TIMED_OUT,
                               G_CALLBACK (port_timed_out_cb),
                               self);
+        g_signal_connect (port,
+                          MM_PORT_SIGNAL_REMOVED,
+                          G_CALLBACK (port_removed_cb),
+                          self);
     }
 
     /* Store kernel device */
@@ -506,6 +527,12 @@ mm_base_modem_release_link_port (MMBaseModem  *self,
     g_autofree gchar *key = NULL;
     MMPort           *port;
 
+    if (!self->priv->link_ports) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Cannot release link port '%s/%s', no link ports table", subsystem, name);
+        return FALSE;
+    }
+
     key  = g_strdup_printf ("%s%s", subsystem, name);
     port = g_hash_table_lookup (self->priv->link_ports, key);
     if (!port) {
@@ -626,6 +653,13 @@ mm_base_modem_wait_link_port (MMBaseModem         *self,
         return;
     }
 
+    if (!self->priv->link_ports) {
+        g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                                 "Cannot wait for port '%s/%s', no link ports table", subsystem, name);
+        g_object_unref (task);
+        return;
+    }
+
     key = g_strdup_printf ("%s%s", subsystem, name);
     port = g_hash_table_lookup (self->priv->link_ports, key);
     if (port) {
@@ -1101,6 +1135,9 @@ mm_base_modem_has_at_port (MMBaseModem *self)
     gpointer value;
     gpointer key;
 
+    if (!self->priv->ports)
+        return FALSE;
+
     /* We'll iterate the ht of ports, looking for any port which is AT */
     g_hash_table_iter_init (&iter, self->priv->ports);
     while (g_hash_table_iter_next (&iter, &key, &value)) {
@@ -1127,6 +1164,11 @@ mm_base_modem_get_port_infos (MMBaseModem *self,
     GArray         *port_infos;
     MMPort         *port;
 
+    if (!self->priv->ports) {
+        *n_port_infos = 0;
+        return NULL;
+    }
+
     *n_port_infos = g_hash_table_size (self->priv->ports);
     port_infos = g_array_sized_new (FALSE, FALSE, sizeof (MMModemPortInfo), *n_port_infos);
     g_hash_table_iter_init (&iter, self->priv->ports);
@@ -1321,6 +1363,14 @@ mm_base_modem_organize_ports (MMBaseModem *self,
     if (self->priv->primary)
         return TRUE;
 
+    /* Ports table is created on init and removed on dispose(), not on
+     * finalize(), so there is a chance this may happen */
+    if (!self->priv->ports) {
+        g_set_error_literal (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                             "No ports table");
+        return FALSE;
+    }
+
     g_hash_table_iter_init (&iter, self->priv->ports);
     while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &candidate)) {
         switch (mm_port_get_port_type (candidate)) {
@@ -1745,6 +1795,7 @@ cleanup_modem_port (MMBaseModem *self,
 
     /* Cleanup on all control ports */
     g_signal_handlers_disconnect_by_func (port, port_timed_out_cb, self);
+    g_signal_handlers_disconnect_by_func (port, port_removed_cb, self);
 
 #if defined WITH_MBIM
     /* We need to close the MBIM port cleanly when disposing the modem object */
